% Compute the stiffness matrices of the individual tetrahedral gcells.
% Return an array of the element matrices so they may be assembled.
%    Call as
% ems = stiffness(feb, x, u)
%     x=geometry field
%     u=displacement field
%
function ems = stiffness (self, x, u)
    gcells = get(self.feblock,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    centroid = ones(nfens,1)/nfens;
    dim = get(x,'dim');
    simvol= 1/factorial(dim);
    % Pre-allocate the element matrices
    ems(1:nfens*ngcells) = deal(elemat);
    iem=1;
    % Material
    mat = get(self.feblock, 'mater');
    D = tangent_moduli(mat,struct([]));
    [N,W]=eig(D);
    [ignore,ix]=sort(diag(W));
    Wmean=(W(ix(1),ix(1))+W(ix(2),ix(2)))/2;
    % we have to define this function but it is not satisfactory
    function B = Blmat(Ndersp,Rm)
        nfn= size(Ndersp,1);
        B = zeros(6,nfn*dim);
        for i= 1:nfn
            B(:,dim*(i-1)+1:dim*i)...
                = [ Ndersp(i,1) 0           0  ; ...
                0           Ndersp(i,2) 0  ; ...
                0           0           Ndersp(i,3) ; ...
                Ndersp(i,2) Ndersp(i,1) 0  ; ...
                Ndersp(i,3) 0           Ndersp(i,1) ; ...
                0           Ndersp(i,3) Ndersp(i,2) ]*Rm';
        end
        return;
    end
    % Make the inverse map from finite element nodes to geometric cells
    f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
    gmap=get (f2gmap,'gcell_map');
    fenodemap=get (f2gmap,'fenodes');
    % Now loop over all finite element nodes
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=fenodemap{nix};
        if ~isempty(gl) % This node has a element patch in this block
            % establish local numbering of all nodes of the patch @node nix
            lnmap =0*(1:length(gmap));
            for k=1:length(gl)
                kconn = get(gcells(gl(k)), 'conn'); % connectivity
                for j= 1:length(kconn)
                    if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
                        lnmap(kconn(j))=kconn(j);
                    end
                end
            end
            patchconn=lnmap(lnmap>0);
            % now store the local numbers
            for k=1:length(patchconn)
                lnmap(patchconn(k)) =k;
            end
            lnmap(thisnn) =length(patchconn)+1;
            c = gather(x, thisnn, 'values', 'noreshape'); % coordinates of central node
            % material tangent
            D = tangent_moduli (mat, struct ('xyz',c));
            Rm = material_directions(self,[],[],c);
            % preallocate the strain-displacement matrix
            Nspatialderavg=zeros((length(patchconn)+1),dim);
            Vpatch =0;
            for k=1:length(gl)
                kconn = get(gcells(gl(k)), 'conn'); % connectivity
                lx = gather(x, kconn, 'values', 'noreshape'); % coordinates of nodes
                lx(:,1)=lx(:,1)-c(1);
                lx(:,2)=lx(:,2)-c(2);
                lx(:,3)=lx(:,3)-c(3);
                Nder = bfundpar(gcells(gl(k)), centroid);
                xl=lx*Rm;% local coordinates
                Nspatialder = bfundsp (gcells(gl(k)), Nder, xl);
                detJ = Jacobian_volume(gcells(gl(k)),centroid,xl);
                Ve = detJ * simvol/nfens;%
                Vpatch=Vpatch+Ve;
                Nspatialderavg(lnmap(kconn),:)=Nspatialderavg(lnmap(kconn),:)+Ve*Nspatialder;
            end
            Nspatialderavg=Nspatialderavg/ Vpatch;
            Bnodal= Blmat (Nspatialderavg,Rm);
            Ke = Bnodal'*D*Bnodal * Vpatch;
            if (self.stabfact>0)
                np=(length(patchconn)+1);
                Phi=zeros(dim*np, 12);
                lx = gather(x, [patchconn thisnn], 'values', 'noreshape'); % coordinates of nodes
                lx(:,1)=lx(:,1)-c(1);
                lx(:,2)=lx(:,2)-c(2);
                lx(:,3)=lx(:,3)-c(3);
                Phi(1:dim:end-1, 1)=1;
                Phi(1:dim:end-1, 2)=lx(:,1)';
                Phi(1:dim:end-1, 3)=lx(:,2)';
                Phi(1:dim:end-1, 4)=lx(:,3)';
                Phi(2:dim:end, 5)=1;
                Phi(2:dim:end, 6)=lx(:,1)';
                Phi(2:dim:end, 7)=lx(:,2)';
                Phi(2:dim:end, 8)=lx(:,3)';
                Phi(3:dim:end, 9)=1;
                Phi(3:dim:end, 10)=lx(:,1)';
                Phi(3:dim:end, 11)=lx(:,2)';
                Phi(3:dim:end, 12)=lx(:,3)';
                Kstab=self.stabfact*Wmean*(eye(size(Ke))-Phi*inv(Phi'*Phi)*Phi');
                Ke = Ke + Kstab;
            end
            ems(iem) = set(ems(iem), 'mat', Ke);
            ems(iem) = set(ems(iem), 'eqnums', gather(u, [patchconn thisnn], 'eqnums'));
            iem=iem+1;
        end
    end
    return;
end


